﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using HeuristicLab.Optimization;

namespace HeuristicLab.Algorithms.TabuSearch {
  [Item("Memory", "A generic memory structure for tabu search.")]
  [StorableClass]
  public class Memory : Item, IMemory {
    [Storable]
    private IDictionary<string, IItem> memory;
    
    [StorableConstructor]
    protected Memory(bool deserializing) : base(deserializing) { }
    protected Memory(Memory original, Cloner cloner)
      : base(original, cloner) {
      memory = new Dictionary<string, IItem>();
      foreach (string key in original.memory.Keys) {
        memory[key] = cloner.Clone(original.memory[key]);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Memory(this, cloner);
    }

    public Memory()
      : base() {
        memory = new Dictionary<string, IItem>();
    }

    public void Put(string key, IItem item) {
      memory[key] = item;
    }

    public IItem Get(string key) {
      if (memory.ContainsKey(key))
        return memory[key];
      else
        return null;
    }

    public void Reset() {
      memory.Clear();
    }
  }
}
