﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.TabuSearch {
  [Item("MemoryCreator", "An operator that creates a new empty memory. It can also replace an existing memory with a new empty one.")]
  [StorableClass]
  public class MemoryCreator : SingleSuccessorOperator {
    public ValueLookupParameter<Memory> MemoryParameter {
      get { return (ValueLookupParameter<Memory>)Parameters["Memory"]; }
    }
    protected ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }

    [StorableConstructor]
    protected MemoryCreator(bool deserializing) : base(deserializing) { }
    protected MemoryCreator(MemoryCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MemoryCreator(this, cloner);
    }

    public MemoryCreator()
      : base() {
      Parameters.Add(new ValueLookupParameter<Memory>("MemoryCreator", "The memory.", new Memory()));
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope."));
    }

    public override IOperation Apply() {
      IVariable var;
      CurrentScope.Variables.TryGetValue(MemoryParameter.ActualName, out var);
      if (var != null)
        var.Value = (IItem)MemoryParameter.ActualValue.Clone();
      else
        CurrentScope.Variables.Add(new Variable(MemoryParameter.ActualName, (IItem)MemoryParameter.ActualValue.Clone()));
      return base.Apply();
    }
  }
}
